import ThemeNotFoundError from './errors/ThemeNotFoundError';
import LayoutNotFoundError from './errors/LayoutNotFoundError';
import AssetsObjectInvalidError from './errors/AssetsObjectInvalidError';
import AssetsLoadError from './errors/AssetsLoadError';
import {findLayout, findTheme} from "./utils";
import AssetsNotFoundError from "./errors/AssetsNotFoundError";

const DESKTOP_HEADER_TOP_OFFSET = 88;

/**
 * Setka Editor assets.
 */
export default class EditorAssets {

    /**
     * @type {Object}
     */
    assets;

    /**
     * @type {Object}
     */
    config;

    /**
     * @type {boolean}
     */
    assetsStatus = false;

    /**
     * @type {Promise}
     */
    assetsPromise;

    /**
     * @type {Error}
     */
    assetsError;

    constructor(config) {
        this.config = { ...config };
        this.config.headerTopOffset = DESKTOP_HEADER_TOP_OFFSET;
    }

    /**
     * Initializing loading JSON config for Setka Editor.
     *
     * @return {Promise}
     */
    startLoading(url) {
        return new Promise((resolve, reject) => {
            fetch(url)
                .then(response => {
                    if(200 === response.status) {
                        return response.json();
                    } else {
                        this.assetsError = AssetsLoadError.create();
                        reject(this.assetsError);
                    }
                })
                .then(fetchedConfig => {
                    this.config = { ...this.config, ...fetchedConfig.config };
                    this.assets = { ...fetchedConfig.assets };
                    this.assetsStatus = true;
                    resolve(fetchedConfig);
                })
                .catch(exception => {
                    this.assetsError = AssetsLoadError.create(exception);
                    reject(this.assetsError);
                })
        });
    }

    /**
     * Finds theme by classname.
     *
     * @param className {string} CSS class name.
     *
     * @throws {Error} If theme not found.
     *
     * @return {object} Theme object.
     */
    getThemeByClassName(className) {
        if('object' !== typeof this.assets.themes) {
            throw new AssetsObjectInvalidError();
        }

        let result = this.assets.themes.find(theme => theme.class_name === className);

        if(result) {
            return result
        }
        throw new ThemeNotFoundError(className);
    }

    /**
     * Finds layout by classname.
     *
     * @param className {string} CSS class name.
     *
     * @throws {Error} If layout not found.
     *
     * @return {object} Layout object.
     */
    getLayoutByClassName(className) {
        if('object' !== typeof this.assets.layouts) {
            throw new AssetsObjectInvalidError();
        }

        let result = this.assets.layouts.find(layout => layout.class_name === className);

        if(result) {
            return result
        }
        throw new LayoutNotFoundError(className);
    }

    /**
     * Finds theme by ID.
     *
     * @param id {string} CSS class name.
     *
     * @throws {Error} If theme not found.
     *
     * @return {object} Theme object.
     */
    getThemeById(id) {
        if('object' !== typeof this.assets.themes) {
            throw new AssetsObjectInvalidError();
        }

        let result = this.assets.themes.find(theme => theme.id === id);

        if(result) {
            return result
        }
        throw new ThemeNotFoundError(id);
    }

    /**
     * Finds layout by ID.
     *
     * @param id {string} CSS class name.
     *
     * @throws {Error} If layout not found.
     *
     * @return {object} Layout object.
     */
    getLayoutById(id) {
        if('object' !== typeof this.assets.layouts) {
            throw new AssetsObjectInvalidError();
        }

        let result = this.assets.layouts.find(layout => layout.id === id);

        if(result) {
            return result
        }
        throw new LayoutNotFoundError(id);
    }

    /**
     * Find Setka Editor assets from content string.
     *
     * @param content {string} HTML markup.
     *
     * @throws {AssetsNotFoundError} If required theme or layout not found in assets storage.
     *
     * @return {Object} Assets
     */
    findAssetsForContent(content) {
        let themeClassName,
            layoutClassName,
            assets = {
                theme: null,
                layout: null,
            };

        try {
            themeClassName  = findTheme(content);
            layoutClassName = findLayout(content);
        } catch(error) {}

        let complexError = new AssetsNotFoundError();

        if(themeClassName) {
            try {
                assets.theme = this.getThemeByClassName(themeClassName).id;
            } catch (error) {
                complexError.theme = error;
            }
        } else {
            complexError.theme = new ThemeNotFoundError('UNKNOWN');
        }

        if(layoutClassName) {
            try {
                assets.layout = this.getLayoutByClassName(layoutClassName).id;
            } catch (error) {
                complexError.layout = error;
            }
        } else {
            complexError.layout = new LayoutNotFoundError('UNKNOWN');
        }

        if(complexError.theme || complexError.layout) {
            this.assetsError = complexError;
            throw complexError;
        } else {
            this.assetsError = null;
        }

        return assets;
    }

    /**
     * @param themeId {string}
     * @param layoutId {string}
     * @return {{
     *     theme: {string},
     *     layout: {string}
     * }}
     */
    findAssetsForIds(themeId, layoutId) {
        let complexError = new AssetsNotFoundError();

        try {
            this.getThemeById(themeId);
        } catch (error) {
            complexError.theme = error;
        }

        try {
            this.getLayoutById(layoutId);
        } catch (error) {
            complexError.layout = error;
        }

        if(complexError.theme || complexError.layout) {
            this.assetsError = complexError;
            throw complexError;
        } else {
            this.assetsError = null;
        }

        return {
            theme: themeId,
            layout: layoutId,
        };
    }

    /**
     * Sets assets promise.
     *
     * @param promise {Promise} A promise which indicates the state of assets loading.
     *
     * @return {this} For chain calls.
     */
    setAssetsPromise(promise) {
        this.assetsPromise = promise;
        return this;
    }

    /**
     * Sets assets status then assets ready to use.
     *
     * @param status {bool} State of assets. True if ready to use.
     *
     * @return {this} For chain calls.
     */
    setAssetsStatus(status) {
        this.assetsStatus = status;
        return this;
    }

    /**
     * Returns assets status.
     *
     * @return {bool} True if assets ready to use.
     */
    getAssetsStatus() {
        return this.assetsStatus;
    }

    /**
     * @return {Error}
     */
    getAssetsError() {
        return this.assetsError;
    }
}
