<?php
namespace Setka\Editor\Service;

use Setka\Editor\Admin\Service\WPQueryFactory;
use Setka\Editor\PostMetas\UseEditorPostMeta;
use Setka\Editor\Service\AMP\AMP;
use Setka\Editor\Service\Config\AMPConfig;

class QuerySniffer
{
    /**
     * @var \WP_Query
     */
    private $main;

    /**
     * @var \WP_Query[]
     */
    private $queries = array();

    /**
     * @var boolean True if Gutenberg available and should be used.
     */
    private $gutenbergSupport;

    /**
     * @var AMP
     */
    private $amp;

    /**
     * @var UseEditorPostMeta
     */
    private $useEditorPostMeta;

    /**
     * @var ScriptStyles
     */
    private $scriptStyles;

    /**
     * QuerySniffer constructor.
     *
     * @param bool $gutenbergSupport
     * @param \WP_Query $mainQuery
     * @param AMP $amp
     * @param UseEditorPostMeta $useEditorPostMeta
     * @param ScriptStyles $scriptStyles
     */
    public function __construct(
        $gutenbergSupport,
        \WP_Query $mainQuery,
        AMP $amp,
        UseEditorPostMeta $useEditorPostMeta,
        ScriptStyles $scriptStyles
    ) {
        $this->queries[]         = $mainQuery;
        $this->main              = $mainQuery;
        $this->gutenbergSupport  = $gutenbergSupport;
        $this->amp               = $amp;
        $this->useEditorPostMeta = $useEditorPostMeta;
        $this->scriptStyles      = $scriptStyles;
    }

    /**
     * Main method.
     */
    public function scan()
    {
        if ($this->isAMP()) {
            $this->ampScan();
        } else {
            $this->regularScan();
        }
    }

    /**
     * Scan queries for AMP request.
     */
    private function ampScan()
    {
        foreach ($this->queries as $query) {
            if (!is_array($query->posts)) {
                continue;
            }

            if ($this->isMainQueryAndNotSingular($query)) {
                continue;
            }

            foreach ($query->posts as $post) {
                $this->amp->setupThemeAndLayoutFromPostMeta($post);
                if ($this->gutenbergSupport) {
                    do_blocks($post->post_content);
                }
            }
        }

        $this->scriptStyles
            ->setAmpCSS($this->amp->setupCSS()->getCss())
            ->setFonts($this->amp->setupFonts()->getFonts())
            ->enableAMPAssets();
    }

    /**
     * Scan queries for regular request.
     */
    private function regularScan()
    {
        foreach ($this->queries as $query) {
            if (!$this->isValidQuery($query)) {
                continue;
            }

            if ($this->isMainQueryAndNotSingular($query)) {
                continue;
            }

            foreach ($query->posts as $post) {
                if (!$this->isValidPost($post)) {
                    continue;
                }

                if ($this->isSetkaPost($post)) {
                    $this->scriptStyles->enableRegularAssets();
                    break 2;
                }
            }
        }
    }

    /**
     * @param \WP_Query $query
     *
     * @return bool
     */
    private function isValidQuery(\WP_Query $query)
    {
        return WPQueryFactory::isValidAndNotEmptyQuery($query);
    }

    /**
     * @param mixed $post
     *
     * @return bool
     */
    private function isValidPost($post)
    {
        return WPPostFactory::isValidPost($post);
    }

    /**
     * @param \WP_Query $query
     *
     * @return bool
     */
    private function isMainQueryAndNotSingular(\WP_Query $query)
    {
        return $this->main === $query && !$query->is_singular();
    }

    /**
     * @param \WP_Post $post
     *
     * @return boolean True if post created (contains) in Setka Editor.
     */
    private function isSetkaPost(\WP_Post $post)
    {
        return $this->useEditorPostMeta->setPostId($post->ID)->get();
    }

    /**
     * Check if we should handle request as AMP.
     * @return bool True if AMP page requested and we should use AMP styles.
     */
    private function isAmp()
    {
        return $this->amp->isAmpSupport() && $this->isAMPEndpoint();
    }

    /**
     * @return bool
     */
    private function isAMPEndpoint()
    {
        return AMPConfig::isAMPEndpoint();
    }

    /**
     * @param \WP_Query $query
     * @return $this
     */
    public function addQuery(\WP_Query $query)
    {
        $this->queries[] = $query;
        return $this;
    }
}
